package com.snowtide.pdf.test;

import com.snowtide.util.logging.Log;
import com.snowtide.util.logging.LoggingRegistry;

import java.io.File;
import java.io.StringWriter;

import org.jpedal.PdfDecoder;
import org.jpedal.exception.PdfException;
import org.jpedal.grouping.PdfGroupingAlgorithms;
import org.jpedal.objects.PdfData;
import org.jpedal.objects.PdfPageData;

/**
 * TestPerformance subclass dedicated to benchmarking the JPedal
 * library.
 *
 * @version ©2004-2007 Snowtide Informatics Systems, Inc.
 */
public class TestJPedalPerformance extends TestPerformance {
    private static final Log log = LoggingRegistry.getLog(TestJPedalPerformance.class);

    public static void main (String[] args) throws Exception {
        PdfDecoder.useTextExtraction();
        new TestJPedalPerformance().runTest(args);
    }

    public String getLibraryId () {
        return "jpedal";
    }

    protected void extractText (File pdfFile, StringWriter sb) throws Exception {
        decodeFile(pdfFile.getAbsolutePath(), sb);
    }

    /**
     * This method pulled from org.jpedal.examples.text.ExtractTextAsWordlist and optimized.
     */
    private static void decodeFile (String file_name, StringWriter sb) throws Exception {
        boolean outputMessages = false;
        PdfDecoder decodePdf;

        //PdfDecoder returns a PdfException if there is a problem
        decodePdf = new PdfDecoder(false);
        
        // extract just text, use default dpi for image data 
        //     (should be ignored since we're in text mode anyway, right?)
        decodePdf.setExtractionMode(PdfDecoder.TEXT); 
        decodePdf.init(true);
        //make sure widths in data CRITICAL if we want to split lines correctly!!

        /**
         * open the file (and read metadata including pages in  file)
         */
        if(outputMessages) System.out.println("Opening file :" + file_name);

        decodePdf.openPdfFile(file_name);

        /**
         * extract data from pdf (if allowed).
         */
        if ((decodePdf.isEncrypted()) && (!decodePdf.isExtractionAllowed())) {
            if(outputMessages){
                System.out.println("Encrypted settings");
                System.out.println("Please look at SimpleViewer for code sample to handle such files");
            }
        } else{
            //page range
            int start = 1, end = decodePdf.getPageCount();

            /**
             * extract data from pdf
             */
            for (int page = start; page < end + 1; page++) { //read pages

                //decode the page
                decodePdf.decodePage(page);

                //get the PdfData object which now holds the content
                PdfData pdf_text = decodePdf.getPdfData();

                /** create a grouping object to apply grouping to data*/
                PdfGroupingAlgorithms currentGrouping =
                        new PdfGroupingAlgorithms(pdf_text);

                /**use whole page size for  demo - get data from PageData object*/
                PdfPageData currentPageData = decodePdf.getPdfPageData();

                int x1 = currentPageData.getMediaBoxX(page);
                int x2 = currentPageData.getMediaBoxWidth(page) + x1;

                int y1 = currentPageData.getMediaBoxY(page);
                int y2 = currentPageData.getMediaBoxHeight(page) + y1;

                //tell user
                if(outputMessages)
                    System.out.println(
                            "Page "+page+" Extracting text from rectangle ("
                            + x1
                            + ","
                            + y1
                            + " "
                            + x2
                            + ","
                            + y2
                            + ")");

                /**Co-ordinates are x1,y2 (top left hand corner), x2,y1(bottom right) */

                /**The call to extract the list*/
                String text =null;

                /**new 7th October 2003 - define punctuation*/
                try{
                    text =currentGrouping.extractTextInRectangle(
                            x1,
                            y2,
                            x2,
                            y1,
                            page,
                            true,
                            true);
                } catch (PdfException e) {
                    decodePdf.closePdfFile();
                    System.err.println("Exception= "+ e+" in "+file_name);
                }

                if (text == null) {
                    if(outputMessages)
                        System.out.println("No text found");

                } else {
                    sb.write(text);
                }

                //remove data once written out
                decodePdf.flushObjectValues(false);

            }

            /**
             * flush data structures - not strictly required but included
             * as example
             */
            decodePdf.flushObjectValues(true); //flush any text data read

            /**tell user*/
            if(outputMessages)
                System.out.println("Text read");

        }

        /**close the pdf file*/
        decodePdf.closePdfFile();

    }
}
