package com.snowtide.pdf.test;

import com.snowtide.util.logging.Log;
import com.snowtide.util.logging.LoggingRegistry;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Iterator;
import java.util.LinkedList;

import com.etymon.pj.Pdf;
import com.etymon.pj.exception.InvalidPdfObjectException;
import com.etymon.pj.exception.PjException;
import com.etymon.pj.object.*;

/**
 * TestPerformance subclass dedicated to benchmarking Etymon's PJ PDF
 * library.
 *
 * @version ©2004-2007 Snowtide Informatics Systems, Inc.
 */
public class TestPJPerformance extends TestPerformance {
    private static final Log log = LoggingRegistry.getLog(TestPJPerformance.class);

    public static void main (String[] args) throws Exception {
        new TestPJPerformance().runTest(args);
    }

    public String getLibraryId () {
        return "pj";
    }

    protected void extractText (File pdfFile, StringWriter sb) throws Exception {
        PjWrapper w = new PjWrapper(pdfFile.getAbsolutePath());
        w.getAllText(sb);
    }

    /**
     * This is a wrapper for the Pj PDF parser.
     */
    class PjWrapper {

        Pdf pdf;
        PjCatalog catalog;
        PjPagesNode rootPage;

        public PjWrapper (String filename) throws IOException, PjException {
            pdf = new Pdf(filename);

            catalog = (PjCatalog)pdf.getObject(pdf.getCatalog());
            rootPage = (PjPagesNode)pdf.resolve(catalog.getPages());
        }

        /**
         * Extracts as much text as possible into the given stringbuffer.
         *
         * NOTE: Pj does not support LZW, so some text in some PDF's may not be accessible.
         */
        public void getAllText (StringWriter sb) throws PjException {
            Iterator streamIter = getContentStreams(getAllPages()).iterator();
            PjStream stream;
            String streamData;
            boolean moreData;
            int textStart, textEnd;

            while (streamIter.hasNext()) {
                stream = (PjStream)streamIter.next();
                // todo -- we're just assuming the streams are flate compressed -- should check what filters are in force, and decode appropriately
                //     (if possible -- it should support ascii85 and hex at the very least)
                stream = stream.flateDecompress();

                streamData = new String(stream.getBuffer());

                moreData = true;
                textStart = textEnd = 0;

                // wow, this is a really stupid way of extracting text....we're just grabbing the character codes out.
                // eh, for our purposes here (determining comparative performance), this gives PJ an advantage, so
                // let it be...
                while (moreData) {
                    if ((textStart = streamData.indexOf('(', textEnd+1))<0) {
                        moreData = false;
                        break;
                    }

                    if ((textEnd = streamData.indexOf(')', textStart+1))<0) {
                        moreData = false;
                        break;
                    }

                    try {
                        sb.write(PjString.decodePdf(streamData.substring(textStart, textEnd+1)));
                    } catch (Exception e) {
                        if (log.isDebugEnabled()) log.debug("Error decoding PDF stream", e);
                        //			System.out.println("malformed string: "+streamData.substring(textStart, textEnd+1));
                    }
                }
            }
        }

        /**
         * Retuns a list of the content streams in the current PDF.
         */
        public LinkedList getContentStreams (LinkedList pages) throws InvalidPdfObjectException {
            LinkedList streams = new LinkedList();
            Iterator pageIter = pages.iterator();
            PjObject contents;

            while (pageIter.hasNext()) {
                contents = pdf.resolve(((PjPage)pageIter.next()).getContents());

                // should only be a stream or an array of streams (or refs to streams)
                if (contents instanceof PjStream)
                    streams.add(contents);
                else {
                    Iterator streamsIter = ((PjArray)contents).getVector().iterator();

                    while (streamsIter.hasNext())
                        streams.add(pdf.resolve((PjObject)streamsIter.next()));
                }
            }

            return streams;
        }

        /**
         * Performs a post-order traversal of the pages tree
         * from the root node.
         */
        public LinkedList getAllPages () throws InvalidPdfObjectException {
            LinkedList pages = new LinkedList();
            getPages(rootPage, pages);
            return pages;
        }

        /**
         * Performs a post-order traversal of the page tree rooted at the given node.
         */
        private void getPages (PjObject node, LinkedList pages) throws InvalidPdfObjectException {
            PjPagesNode pageNode = null;

            // let's hope pdf's don't have pointers to pointers
            if (node instanceof PjReference) {
                pageNode = (PjPagesNode)pdf.resolve(node);
            } else {
                pageNode = (PjPagesNode)node;
            }

            if (pageNode instanceof PjPage) {
                pages.add(pageNode);
                return;
            }

            Iterator iter = ((PjArray)((PjPages)pageNode).getKids()).getVector().iterator();

            while (iter.hasNext()) {
                getPages((PjObject)iter.next(), pages);
            }
        }
    }
}
