package com.snowtide.pdf.test;

import com.snowtide.PDF;
import com.snowtide.pdf.Document;
import com.snowtide.pdf.OutputTarget;
import com.snowtide.util.logging.Log;
import com.snowtide.util.logging.LoggingRegistry;

import java.io.*;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

/**
 * This is a simple class for running performance benchmarks on PDFTextStream on a set of PDF files.
 *
 * The main(String[]) method should be called with a list of paths to PDF files to be benchmarked.
 * Processing time results are outputted to stdout in the following format:
 *
 * (/[digits]ms=[name of pdf file]\n)*
 *
 * This report always provides the best time for each pdf file noted over four runs.
 * 
 * [digits] may also be the following reserved values to indicate exceptional results:
 *    -1: If hard error prevented the text of the PDF file from being extracted.
 *    -2: If the extraction process took more than 60 seconds (any processing runs that last 60 seconds
 *          are killed, with the assumption that an infinite loop has been hit -- common in some PDF libraries)
 *
 * @version ©2004-2007 Snowtide Informatics Systems, Inc.
 */
public class TestPerformance  {
    private static final Log log = LoggingRegistry.getLog(TestPerformance.class);

    static {
        Logger.getRootLogger().removeAllAppenders();
        Logger.getRootLogger().addAppender(new ConsoleAppender(new PatternLayout("%p [%t] %c(%L): %m%n"), "System.out"));
        Logger.getRootLogger().setLevel(Level.INFO);
    }

    public static void main (String[] args) throws Exception {
        new TestPerformance().runTest(args);
    }

    public final void runTest (String[] pdfFiles) throws Exception {
        long time;
        final long[] tmp = new long[1];
        final boolean[] error = new boolean[1];
        
        final Thread mainThread = Thread.currentThread();

        for (int i = 0, len = pdfFiles.length; i<len; i++) {
            final File pdfFile = new File(pdfFiles[i]);

            if (pdfFile.exists() && pdfFile.canRead()) {
                time = tmp[0] = Integer.MAX_VALUE;
                error[0] = false;
                
                for (int f = 0; f<4; f++) {
                    // running each PDF file 3 times and taking the best timing -- 
                    // this eliminates any noise from classloading, special intialization routines,
                    // and some protection against a timing going awry because CPU taken
                    // by other background processes on the box doing the benchmark
                    
                    Thread t = new Thread (new Runnable () {
                        public void run () {
                            try {
                                StringWriter sb = new StringWriter(1024);
                                tmp[0] = System.currentTimeMillis();
                                extractText(pdfFile, sb);
                                tmp[0] = System.currentTimeMillis() - tmp[0];
                                
                                writeText(getLibraryId(), pdfFile.getName(), sb.toString());
                            } catch (Exception e) {
                                log.error("Error reading pdf file ["+pdfFile.getAbsolutePath()+"]", e);
                                error[0] = true;
                            }
                            
                            mainThread.interrupt();
                        }
                    });
                    t.start();
                    
                    try {
                        mainThread.sleep(60000);
                        
                        log.warn("Had to kill the worker thread testing pdf [" + pdfFile.getName() + "] using " + getLibraryId() +
                                " -- took more than 1 minute, have to assume infinite loop.");
                        
                        // this is Not Good (tm), but it's really the only option we have since we're dealing with infinite loops generated by other libraries
                        // we haven't seen wacky behaviour due to this function being used in conjunction with pdfbox, jpedal, and pj, so it looks like
                        // either (a) they don't use stateful global objects that the stop function would cause to be in an inconsistent state, or (b)
                        // we've just been lucky (unlikely given how much benchmarking we've done)
                        t.stop();
                        time = -2;
                        break;
                    } catch (InterruptedException e) {
                        // worker thread finished...check for errors
                        if (error[0]) {
                            time = -1;
                            break;
                        }
                    }
                    
                    if (tmp[0] < time) time = tmp[0];
                }
                
                System.out.println("/" + time + "ms=" + pdfFile.getName());
                
                // these tests are primarily run through an external python script, so get the data back to it asap...
                System.out.flush();
            } else {
                log.error("PDF file ["+pdfFile.getAbsolutePath()+"] does not exist or cannot be read.");
            }
        }
    }

    public String getLibraryId () {
        return "pdfts";
    }

    /**
     * Reads the text out of the given pdf file into the provided StringWriter.
     *
     * A stringwriter is used instead of a stringbuffer because the PDFBox library requires a Writer
     * subclass for text extraction -- the other libraries can adapt to that restriction pretty easily.
     */
    protected void extractText (File pdfFile, StringWriter sb) throws Exception {
        OutputTarget tgt = new OutputTarget(sb);
        Document stream = PDF.open(pdfFile);
        stream.pipe(tgt);

        stream.close();
    }

    /**
     * Writes the given text string to a file in the tmp directory identifiable by a prefix consisting of the given
     * library id and pdfName.
     */
    static void writeText (String libId, String pdfName, String text) throws IOException {
        File f = File.createTempFile(libId+pdfName, ".tmp");
        OutputStreamWriter out = new OutputStreamWriter(new FileOutputStream(f));
        out.write(text);
        out.flush();
        out.close();
    }
}
